#!/bin/sh
# $Id: t-pmilter-3.sh,v 1.16 2011/02/11 19:29:47 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test pmilter returning capabilities (turn off some features)
# uses smtpcnf.sh.
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

chkcto smtps MTA_USE_PMILTER '(--enable-pmilter)'

OUT="t-pmilter-0.out"
PMPRG="-p ../libpmilter/t-pmilter-1"
PMOPT=""

if test X"`uname`" != "XOSF1"
then
while getopts O:p:s:V FLAG
do
  case "${FLAG}" in
    O) PMOPT="${PMOPT} ${OPTARG}";;
    p) PMPRG="-p ${OPTARG}";;
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || myexit 1
rm -f ${MTMAP}

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 12; nameserver = ${NS}; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }

smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
wait_for_server=4;
policy_milter { socket {type=unix; path="pmilter.sock";} timeout=2; }
}
EOF


# ----------------------------------------
# would reject mail command in milter but MAIL is disabled
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r m=550 -C M" -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'MAIL=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${GREP} -i 'err' ${PML} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject rcpt command in milter
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r r=550 -C R" -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'RCPT=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${GREP} -i 'err' ${PML} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject DATA command in milter
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r d=550 -C D" -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DATA=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${GREP} -i 'err' ${PML} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

# ----------------------------------------
# reject final dot command in milter
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r b=550 -C B" -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"a@b.c" -R"x@y.z"
then
    ERR=false
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && myexit 1
fi
if ${GREP} 'DOT=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${GREP} -i 'err' ${PML} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && myexit 1
else
    ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

## # ----------------------------------------
## # discard mail (body)
## TEST=7
## if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
## then
## 
## if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r b=90" -Z 'smtpc2' -A 1 -E 0 -X 0 "$@" -S"s${TEST}@b.c" -R"r${TEST}@y.z"
## then
##     ERR=false
## else
##     ERR=true
##     echo "$0: test ${TEST} failed"
## fi
## 
## # check whether there's a mail body
## rm -f ${OUT}
## ls ?/S* > ${OUT} 2>/dev/null
## if test -s ${OUT}
## then
##     ERR=true
##     echo "$0: test ${TEST} failed: found mail in queue"
##     ${VERBOSE} && ${CAT} ${OUT}
## fi
## 
## if ${ERR}
## then
##     echo "$0: test ${TEST} check failed"
##     ERRS=`expr ${ERRS} + 1 `
##     test X"${MTA_STOPONERROR}" != X && myexit 1
## else
##     ${VERBOSE} && echo "test ${TEST} succeeded"
## fi
## fi
## 
## 
## # ----------------------------------------
## # discard recipient
## TEST=8
## if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
## then
## 
## if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r r=90" -Z 'smtpc2' -A 1 -E 0 -X 0 "$@" -S"s${TEST}@b.c" -R"r${TEST}@y.z"
## then
##     ERR=false
## else
##     echo "$0: test ${TEST} failed"
##     ERR=true
## fi
## 
## # check whether there's a mail body
## rm -f ${OUT}
## ls ?/S* > ${OUT} 2>/dev/null
## if test -s ${OUT}
## then
##     ERR=true
##     echo "$0: test ${TEST} failed: found mail in queue"
##     ${VERBOSE} && ${CAT} ${OUT}
## fi
## 
## if ${ERR}
## then
##     echo "$0: test ${TEST} check failed"
##     ERRS=`expr ${ERRS} + 1 `
##     test X"${MTA_STOPONERROR}" != X && myexit 1
## else
##     ${VERBOSE} && echo "test ${TEST} succeeded"
## fi
## fi
## 
## 
## # ----------------------------------------
## # discard sender
## TEST=9
## if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
## then
## 
## if ${SHELL} ${SCRIPTS}/smtpcnf.sh ${PMPRG} ${PMOPT} -P "-r m=90" -Z 'smtpc2' -A 1 -E 0 -X 0 "$@" -S"s${TEST}@b.c" -R"r${TEST}@y.z"
## then
##     ERR=false
## else
##     ERR=true
##     echo "$0: test ${TEST} failed"
## fi
## 
## # check whether there's a mail body
## rm -f ${OUT}
## ls ?/S* > ${OUT} 2>/dev/null
## if test -s ${OUT}
## then
##     ERR=true
##     echo "$0: test ${TEST} failed: found mail in queue"
##     ${VERBOSE} && ${CAT} ${OUT}
## fi
## 
## if ${ERR}
## then
##     echo "$0: test ${TEST} check failed"
##     ERRS=`expr ${ERRS} + 1 `
##     test X"${MTA_STOPONERROR}" != X && myexit 1
## else
##     ${VERBOSE} && echo "test ${TEST} succeeded"
## fi
## fi


# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${OUT}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  myexit 1
fi
