#!/bin/sh
# $Id: t-reuse-3.sh,v 1.4 2013/11/23 19:59:43 ca Exp $
#
# ----------------------------------------
# "negative" test of session reuse:
# don't set keep_open=1 for addresses in a list that can be delivered
# within the same (DA) transaction.
# uses smtpcnf.sh
# ----------------------------------------
#

# source directory of this (and other) shell script
SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

killit()
{
rm -f ${QMGRCONFMAP} ${MTMAP}
exit 1
}

# create restrictions
${CAT} > ${QMGRCONF} <<EOF
oci:127.0.0.1	1
ocm:127.0.0.1	1
octo:127.0.0.1	20
EOF
# create map
rm -f ${QMGRCONFMAP}
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}
rm -f ${MTMAP}
trap killit 2

# create mailertable (save old version?)
${CAT} > ${MT} <<EOF
local.dom	lmtp:
some.dom	[127.0.0.2]
rcpt.tld	[127.0.0.1]
sender.tld	[10.1.1.1]
EOF

# create aliases
${CAT} > ${ALI} <<EOF
postmaster:	local:
list0:     <user0@rcpt.tld> <user1@rcpt.tld> <user2@some.dom>
owner-list0:     local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}


# ----------------------------------------
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=1;
  flags = { reuse_connection };
  retry_delay_min = 30;
  retry_delay_max = 60;
}
smar { Log_Level = 9; nameserver = ${NS}; DNS_timeout = 5; }
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4;
  connect_only_to = 127.0.0.1;
  flags=timing;
}
smtps { Log_Level = 9;
  daemon_address = localhost:${SRVPORT};
  wait_for_server=4;
}
EOF

# ----------------------------------------
# 
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false

# XREF: QMGR_TEST_SCHED_DLY
if ${SHELL} ${SD}/smtpcnf.sh -E 3 -X '[12]' -1 -Q '-t 0x200 -x10.5' -s 15 -Y '-@ P -w -1' -Z smtpc2 -c '-Fsender.tld -Rlist0@local.dom'
then
  if ${GREP} '^Sessions reused *[1-9]' ${SCL} >/dev/null && ERR=true
  then
    ${VERBOSE} && echo "${TEST}: Sessions reused wrong"
    ERR=true
  fi
  if ${GREP} 'keep_open=1' ${QML} >/dev/null && ERR=true
  then
    ${VERBOSE} && echo "${TEST}: keep_open wrong"
    ERR=true
  fi
else
  ERR=true
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
else
  ${VERBOSE} && echo "test ${TEST} succeeded"
fi
fi

rm -f ${QMGRCONFMAP} f ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
