/*
 * Copyright (c) 2010 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-rsd-0.c,v 1.4 2011/01/30 21:49:10 ca Exp $")
#include "sm/sysexits.h"
#include "sm/str.h"
#include "sm/actdb-int.h"
#define QMGR_DEBUG_DEFINE 1
#include "sm/qmgrdbg.h"
#include "sm/io.h"
#include "sm/mta.h"
#include "sm/test.h"

static int Verbose = 0;

typedef struct sm_ad_S	sm_ad_T;

struct sm_ad_S
{
	const char *sm_ad_a;
	const char *sm_ad_d;
};

sm_ad_T sm_ad[] = {
	{ "<local@meta1.org>",	"meta1.org" },
	{ "<user\\@some@meta1.org>",	"meta1.org" },
	{ "<local@some.meta1.org>",	"some.meta1.org" },
	{ "<local@tld>",	"tld" },
	{ "<local@d>",	"d" },
	{ NULL,	NULL }
};

static sm_ret_T
rsd(void)
{
	sm_ret_T ret;
	int i;
	aq_rcpt_P aq_rcpt;
	sm_str_P dd;
	const char *a, *d;

	ret = aq_rcpt_new(&aq_rcpt);
	SM_TEST_ERR(ret);
#define MTA_DD "default.domain"
	dd = sm_str_scpy(NULL, MTA_DD, sizeof(MTA_DD) + 2);
	SM_TEST_E(dd != NULL);
	for (i = 0; sm_ad[i].sm_ad_a != NULL; i++) {
		a = sm_ad[i].sm_ad_a;
		d = sm_ad[i].sm_ad_d;
		SM_STR_FREE(aq_rcpt->aqr_pa);
		aq_rcpt->aqr_pa = sm_str_scpy(NULL, a, strlen(a) + 2);
		SM_TEST_E(aq_rcpt->aqr_pa != NULL);

		ret = aq_rcpt_set_domain(aq_rcpt, dd);
		SM_TEST_E(sm_is_ok(ret));

		SM_TEST(strcmp(d, sm_str_getdata(aq_rcpt->aqr_domain)) == 0);
		SM_STR_FREE(aq_rcpt->aqr_domain);
	}

	SM_STR_FREE(aq_rcpt->aqr_pa);
	a = "<postmaster>";
	aq_rcpt->aqr_pa = sm_str_scpy(NULL, a, strlen(a) + 2);
	SM_TEST_E(aq_rcpt->aqr_pa != NULL);
	ret = aq_rcpt_set_domain(aq_rcpt, dd);
	SM_TEST_E(sm_is_ok(ret));
	d = MTA_DD;
	SM_TEST(strcmp(d, sm_str_getdata(aq_rcpt->aqr_domain)) == 0);

	return ret;

  error:
	return ret;
}


static void
usage(const char *prg)
{
	sm_io_fprintf(smioerr, "usage: %s [options]\n", prg);
	sm_io_fprintf(smioerr, "Test rcpt domain extraction\n");
	sm_io_fprintf(smioerr, "options:\n");
	sm_io_fprintf(smioerr, "-V    increase verbosity\n");
	exit(EX_USAGE);
}

int
main(int argc, char **argv)
{
	int c;
	sm_ret_T r;
	char *prg;

	prg = argv[0];
	while ((c = getopt(argc, argv, "V")) != -1) {
		switch (c) {
		  case 'V':
			++Verbose;
			break;
		  default:
			usage(prg);
			return EX_USAGE;
		}
	}

	sm_test_begin(argc, argv, "test recipient domain extraction 0");
	r = rsd();
	return sm_test_end();
}
