/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: rcbputipaddr.c,v 1.1 2013/09/12 01:30:46 ca Exp $")
#include "sm/limits.h"
#include "sm/assert.h"
#include "sm/magic.h"
#include "sm/memops.h"
#include "sm/rpool.h"
#include "sm/str-int.h"
#include "sm/rcb.h"
#include "sm/net.h"

/*
**  SM_RCB_PUTIPADDR -- Write ipaddr to rcb
**
**	Parameters:
**		rcb -- sm_rcb_P object to write to
**		rt -- record type
**		ipaddr -- ipaddr to write
**
**	Returns:
**		usual sm_error code; SM_E_OVFLW_NS, EPROTONOSUPPORT
*/

sm_ret_T
sm_rcb_putipaddr(sm_rcb_P rcb, uint32_t rt, sm_ipaddr_P ipaddr)
{
	sm_ret_T ret;
	int len;
	uchar *buf;

	SM_IS_RCB(rcb);
	SM_REQUIRE(ipaddr != NULL);

	/* family, port, addr? */
	switch (ipaddr->ipa_family) {
	  case AF_INET:
		len = sizeof(ipaddr->ipa_addr.ipv4_addr);
		buf = (uchar *)&ipaddr->ipa_addr.ipv4_addr;
		break;

#if MTA_NETINET6
	  case AF_INET6:
		len = sizeof(ipaddr->ipa_addr.ipv6_addr);
		buf = (uchar *)&ipaddr->ipa_addr.ipv6_addr;
		break;
#endif
	  default:
		ret = sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
		goto error;
	}
	ret = sm_rcb_put3uint32(rcb, len + 4, rt, ipaddr->ipa_family);
	if (sm_is_err(ret))
		goto error;
	ret = sm_rcb_putn(rcb, buf, len);
	if (sm_is_err(ret))
		goto error;

	return SM_SUCCESS;

  error:
	return ret;
}
